@@ -184,8 +184,9 @@ class Eurlex:
         >>> eur.make_query(resource_type = "manual", manual_type = "SWD") # doctest: +ELLIPSIS
         PREFIX ...
         """
-        spinner = Halo(text="Appending query text...", spinner="line")
-        spinner.start()
+        if __name__ == "__main__":
+            spinner = Halo(text="Appending query text...", spinner="line")
+            spinner.start()
         assert resource_type is not None, "resource_type must be specified"
         assert resource_type in get_args(
             self._RESOURCE_TYPES
@@ -468,8 +469,8 @@ class Eurlex:
         if limit and limit is not None and isinstance(limit, int):
             query += " limit " + str(limit)
         # somehow this was added in R: FILTER not exists{?work cdm:do_not_index \"true\"^^<http://www.w3.org/2001/XMLSchema#boolean>}. }
-
-        spinner.stop()
+        if __name__ == "__main__":
+            spinner.stop()
         # TODO add formatting option from server format=application%2Fsparql-results%2Bjson (from https://publications.europa.eu/webapi/rdf/sparql)
         # some postprocessing to get rid of newlines
         return query.replace("\n", "")  # .replace("\t", " ")
@@ -497,9 +498,9 @@ class Eurlex:
         >>> eur = Eurlex()
         >>> eur.query_eurlex("PREFIX dcat: <http://www.w3.org/ns/dcat#> PREFIX odp:  <http://data.europa.eu/euodp/ontologies/ec-odp#> PREFIX dct: <http://purl.org/dc/terms/> PREFIX xsd: <http://www.w3.org/2001/XMLSchema#> PREFIX foaf: <http://xmlns.com/foaf/0.1/> SELECT * WHERE { ?d a dcat:Dataset } LIMIT 10")
         """
-
-        spinner = Halo(text="Querying EU SPARQL endpoint ...", spinner="line")
-        spinner.start()
+        if __name__ == "__main__":
+            spinner = Halo(text="Querying EU SPARQL endpoint ...", spinner="line")
+            spinner.start()
         # TODO rename columns - also check for date with regex such as [0-9]{4}\-[0-9]+?\-[0-9]+$
         data_frame = pd.DataFrame()
         # sparql.setReturnFormat(JSON)
@@ -508,7 +509,8 @@ class Eurlex:
             data_frame = sparql_dataframe.get(endpoint, query)
         except Exception as e:
             print("There was an error when performing the query: ", e)
-        spinner.stop()
+        if __name__ == "__main__":
+            spinner.stop()
         return data_frame
 
     notice_type: Literal = ["tree", "branch", "object"]
@@ -518,7 +520,7 @@ class Eurlex:
     def download_xml(
         self,
         url: str,
-        notice: notice_type = "object",
+        notice: notice_type,
         filename: str = None,
         languages: list = ["en", "fr", "de"],
         mode: str = "wb",
@@ -609,17 +611,18 @@ class Eurlex:
             file_content
         )  # TODO alternatively, offer to return instead of saving to file (or make separate function)
 
-    data_type: Literal = ["title", "text", "id", "notices"]
+    data_types: Literal = ["title", "text", "ids", "notice"]
 
     "Download data/documents from EU Cellar based on a given resource URL"
 
     def get_data(
         self,
         url,
-        data_type: data_type,
+        data_type: data_types,
         notice: notice_type = None,
         languages: list = ["en", "fr", "de"],
         include_breaks: bool = False,
+        extract_caselaw_metadata: bool = False,
     ):
         """This function takes a URL or Celex number and returns data, such as the title, text, the id, or notices.
         Parameters
@@ -627,7 +630,7 @@ class Eurlex:
         url
             The URL or CELEX number to download/access
         data_type
-            The data type to download. Valid options are title, text, id or notices. This parameter is required.
+            The data type to download. Valid options are title, text, ids or notice. This parameter is required.
         notice
             The type of notice to download.
         languages
@@ -636,6 +639,9 @@ class Eurlex:
         include_breaks
             Whether or not to insert page breaks into text.
             Default: False
+        extract_caselaw_metadata
+            For the title, tries to break it down into case name, parties and case number.
+            Default: False
         Returns
         -------
             out: The relevant response as str
@@ -653,13 +659,16 @@ class Eurlex:
             data_type
         ), "The type of the data to be parsed is necessary"  # TODO - maybe just parse all in one go?
         assert (
-            data_type in self.data_type
-        ), "The type of data to be parsed has to be one of title, text, id or notices"
-        assert (
-            data_type == "notice"
-            and notice is not None
-            or data_type != notice  # Is this comparison correct?
-        ), "The type of notice to be processed has to be provided"
+            data_type in self.data_types
+        ), "The type of data to be parsed has to be one of title, text, ids or notice"
+        if data_type == "notice":
+            assert (
+                notice is not None and notice in self.notice_type
+            ), "The type of notice to be processed has to be provided"
+        if data_type != "title":
+            assert (
+                extract_caselaw_metadata is False
+            ), "Case law metadata can only be extracted from titles (of caselaw)"
         # TODO
         # Ok, it is a bit weird to filter language not in CELLAR but in http header
         language_header = ""
@@ -673,19 +682,23 @@ class Eurlex:
             else:
                 print("Only three languages at a time are supported")
 
-        print("The language header is: {}".format(language_header))
         if url[:4] == "http" and re.fullmatch(".*cellar.*", url):
-            print("Assuming URL to be a valid, http based EU Cellar resource")
+            if __name__ == "__main__":
+                print("Assuming URL to be a valid, http based EU Cellar resource")
         else:
-            # TODO - Add additional testing?
-            # if (stringr::str_detect(url,"celex.*[\\(|\\)|\\/]")){
-            # assume it is a CELEX number
-            url = "http://publications.europa.eu/resource/celex/" + url
-            print("The CELEX url is: {}".format(url))
+            if not url[:4] == "http":
+                # TODO - Add additional testing?
+                # if (stringr::str_detect(url,"celex.*[\\(|\\)|\\/]")){
+                # assume it is a CELEX number
+                url = "http://publications.europa.eu/resource/celex/" + url
+                if __name__ == "__main__":
+                    print("The CELEX url is: {}".format(url))
 
         if data_type == "title":
+            out = ""
             try:
-                print("Getting title data...")
+                if __name__ == "__main__":
+                    print("Getting title data...")
                 response = requests.get(
                     url,
                     headers={
@@ -696,15 +709,30 @@ class Eurlex:
             except Exception as e:
                 print("There was an error during data retrieval: {}", e)
             if response.status_code == 200:
-                html = BeautifulSoup(response.text, "xml.parser")
-                out += (
-                    html.find_next("//EXPRESSION_TITLE").find_next("VALUE").get_text()
-                )
+                html = BeautifulSoup(response.text, "xml")
+                out = str(html.find("EXPRESSION_TITLE").get_text())
+                if extract_caselaw_metadata and "#" in out:
+                    title = out.split("#")[0].strip()
+                    parties = out.split("#")[1].strip().strip(".")
+                    case_number = out.split("#")[2].strip().strip(".")
+
+                    out = {
+                        "title": title,
+                        "parties": parties,
+                        "case_number": case_number,
+                    }
+                if __name__ == "__main__":
+                    print(out)
             else:
-                print("No content retrieved: {}", response.status_code)
-        if data_type == "text":
+                if __name__ == "__main__":
+                    print("No content retrieved: {}", response.status_code)
+                out += str(response.status_code)
+
+        elif data_type == "text":
+            out = ""
             try:
-                print("Getting text data...")
+                if __name__ == "__main__":
+                    print("Getting text data...")
                 response = requests.get(
                     url,
                     headers={
@@ -717,8 +745,10 @@ class Eurlex:
                 print("There was an error during gathering data: {}", e)
 
             if response.status_code == 200:
-                print("Got a {} reponse, great!".format(response.status_code))
+                if __name__ == "__main__":
+                    print("Got a {} reponse, great!".format(response.status_code))
                 out = self.read_data(response)
+
             elif response.status_code == 300:
                 html = BeautifulSoup(response.content, "html.parser")
                 links_html = html.find_all("a", href=True)
@@ -737,7 +767,7 @@ class Eurlex:
                         },
                     )
                     if multiresponse.status_code == 200:
-                        print(multiresponse.status_code)
+                        print(str(multiresponse.status_code))
                         print(multiresponse.text)
                         multiout += (
                             self.read_data(multiresponse) + "---documentbreak---"
@@ -747,14 +777,21 @@ class Eurlex:
                 print(multiout)
                 out = multiout
             elif response.status_code == 406:
-                out += "NaN"
-                print("missingdoc")
+                out += "NaN" + str(
+                    response.status_code
+                )  # TODO ok this is a pretty ... thing to do
+                if __name__ == "__main__":
+                    print("missingdoc")
             else:
-                print("No content retrieved {}", response)
+                if __name__ == "__main__":
+                    print("No content retrieved {}", response)
             if not include_breaks:
-                out.replace("---documentbreak---", "").replace("---pagebreak---", "")
+                out = out.replace("---documentbreak---", "").replace(
+                    "---pagebreak---", ""
+                )
 
-        if data_type == "ids":
+        elif data_type == "ids":
+            out = ""
             response = requests.get(
                 url,
                 headers={
@@ -763,11 +800,18 @@ class Eurlex:
                 },
             )
             if response.status_code == 200:
-                xml = BeautifulSoup(response.content, "xml.parser")
-                out = xml.find_all(".//VALUE").get_text()
+                xml = BeautifulSoup(response.content, "xml")
+                out = xml.find_all("VALUE")
+                temp = []
+                for v in out:
+                    temp.append(v.get_text())
+                out = temp
+                if __name__ == "__main__":
+                    print(out)
             else:
-                out += response.status_code
-        if data_type == "notice":
+                out += str(response.status_code)
+        elif data_type == "notice":
+            out = ""
             accept_header = "application/xml; notice=" + notice
             if (
                 notice == "object"
@@ -782,15 +826,24 @@ class Eurlex:
                     },
                 )
             if response.status_code == 200:
-                print("Retrived notice successfully.")
+                if __name__ == "__main__":
+                    print("Retrived notice successfully.")
+                    print(response.text)
                 out += response.text
             else:
-                print(f"Something might have gone wrong: {response.status_code}")
-        return out
+                out = str(response.status_code)
+                if __name__ == "__main__":
+                    print(f"Something might have gone wrong: {response.status_code}")
+        else:
+            return "You should not be here."
+        if out:
+            return out
+        else:
+            return 1
 
     # Reads response data and processes it to get the text, based on the content type
     def read_data(self, response):
-        """This fucntion takes a response object, and returns text as a string. This text is parsed from a html, or a pdf. MS Word is not supported for now.
+        """This function takes a response object, and returns text as a string. This text is parsed from a html, or a pdf. MS Word is not supported for now.
         (the doc test currently only tests this function indirectly, as it is called from get_data(), but for testing it separately in doctest quite some things would have to be changed)
         Parameters
         ----------
